#ifndef __PROBABILITY_DISTRIBUTION__
#define __PROBABILITY_DISTRIBUTION__

#include <distribution.h>

//! Verteilung mit gegebenen Wahrscheinlichkeiten.
/**
 * Diese Klasse beschreibt eine Verteilung, die ber vorzugebene
 * Wahrscheinlichkeiten fr die einzelenen Elemente realisiert wird.
 * Die Spezifikation erfolgt ber einen String, der durch Aufruf
 * von init(scope) ausgelesen wird. Der Aufruf von choose() liefert
 * die einzelnen Elemente mit den spezifizierten Wahrscheinlichkeiten. 
 * Der Aufruf von remove_last() sorgt dafr, dass die Wahrscheinlichkeiten 
 * der verbliebenen Elemente entsprechend normiert werden.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class ProbabilityDistribution : public Distribution
{
	public:
 		//! Standard-Konstruktor.
		ProbabilityDistribution() : Distribution(), stringToParse(NULL) {}
		
		//! Implementierung der abstrakten Basisklasse XMLElement.
		/**
		 * Der relevante Teil des XML-Elements wird untersucht und die
		 * zu verwendenden Wahrscheinlichkeiten initialisiert.
		 *
		 * @param scope XML-Element zu diesem Factory-Element.
		 * @return true
		 */
          	bool init(const DOMElement* scope=NULL);
	
	protected:

		//! Methode zum Parsen der Wahrscheinlichkeiten.		 
		/** 
		 * Diese Methode wird verwendet, um einen String mit Wahrscheinlichkeiten
		 * zu parsen und anhand der Werte eine Verteilung zu bestimmen.
		 *
		 * @return 0.
		 */
		int parseString();
		
	private:

		//! String mit Wahrscheinlichkeiten
		char* stringToParse;
};

#endif
