#ifndef __UNIFORM_DISTRIBUTION__
#define __UNIFORM_DISTRIBUTION__

#include <distribution.h>

//! Uniforme Verteilung.
/**
 * Diese Klasse beschreibt eine Gleichverteilung der Elemente aus
 * [0,DistElementsLeft()-1], d.h. der Aufruf von choose() liefert ein
 * jedes der (noch) zu verteilenden Elemente mit gleicher Wahrscheinlichkeit.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class UniformDistribution : public Distribution
{
	public:
 		//! Standard-Konstruktor.
		UniformDistribution() : Distribution() {}
		
		//! Implementierung der abstrakten Basisklasse XMLElement.
		/**
		 * Es wird lediglich das bergebene Argument gesetzt und true
		 * zurueckgeliefert.
		 *
		 * @param scope XML-Element zu diesem Factory-Element.
		 * @return true
		 */
          	bool init(const DOMElement* scope=NULL);

          	//! Re-Implementierung der abstrakten Basisklasse Distribution.
		/**
		 * Der Aufruf von choose() liefert mit gleicher Wahrscheinlichkeit
		 * ein noch nicht gezogenes Element aus [0,distElements()-1]. Der
		 * anschlieende Aufruf von remove_last sorgt dafr, dass dieses
		 * Element bis zum nchsten Aufruf von reset() nicht mehr gezogen 
		 * werden kann.
		 * 
		 * @return Ein noch nicht geogenes Element aus [0,distElements()-1].
		 */
          	int choose();

		//! Re-Implementierung der abstrakten Basiklasse Distribution.
		/**
		 * Das zuletzt gezogene Element wird aus Liste der zu ziehenden 
		 * Elemente entfernt.
		 */
		void remove_last();
		
		//! Re-Implementierung der (abstrakten) Basisklasse Distribution.
		/**
		 * Setzt die internen Strukturen zur Verwaltung bereits gezogener
		 * Elemente zurck.
		 */
		void reset();

		//! Re-Implementierung der (abstrakten) Basisklasse Distribution.
		/**
		 * @return 1/DistElementsLeft() fr noch zu ziehende Elemente
		 * und 0 fr alle anderen.
		 */
		double probability(unsigned int distelem) const;
};

#endif
