#include <gsl/gsl_multifit.h>
#include<gsl/gsl_linalg.h>
#include <gsl/gsl_math.h>

#include <iostream>

void leastSquares(const double *x_data, const double *y_data, unsigned int n, unsigned int d, double *coeff)
{
	double chisq;
	unsigned int m = d+1;
	
	// views
	gsl_vector_const_view y = gsl_vector_const_view_array(y_data, n);
	gsl_vector_view c = gsl_vector_view_array(coeff, m);

	// alloc
	gsl_matrix *X = gsl_matrix_alloc(n, m);
	gsl_matrix *C = gsl_matrix_alloc(m, m);	
	gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc (n, m);
	
	for(unsigned int i=0; i<n; i++)
	{
		for(unsigned int j=0; j<=d; j++)
		{
			gsl_matrix_set(X, i, j, gsl_pow_int(x_data[i],j));
		}
	}

	// least-squares
	gsl_multifit_linear (X, &y.vector, &c.vector, C, &chisq, work);

	// free
	gsl_multifit_linear_free(work);
	gsl_matrix_free(X);
	gsl_matrix_free(C);
}

void leastSquares(const double *x_data, const double *y_data, unsigned int n, unsigned int d, double *coeff, double *matrix)
{
	int s;
	unsigned int i, j, k;
	unsigned int m = d+1;
	
	// views
	gsl_matrix_view C = gsl_matrix_view_array(matrix, m, n);
	
	// alloc
	gsl_matrix *Xt= gsl_matrix_alloc(m, n);
	gsl_matrix *XtX=gsl_matrix_alloc(m, m);
	gsl_matrix *I = gsl_matrix_alloc(m, m);
	gsl_permutation *p = gsl_permutation_alloc(m);
	
	// Xt
	for(i=0; i<n; i++)
	{
		for(j=0; j<m; j++)
		{
			gsl_matrix_set(Xt, j, i, gsl_pow_int(x_data[i],j));
		}
	}

	// XtX = Xt*X
	for(i=0; i<m; i++)
	{
		for(j=0; j<m; j++)
		{
			double sum = 0.0;
			for(k=0; k<n; k++)
			{
				sum += gsl_matrix_get(Xt, i, k) * gsl_matrix_get(Xt, j, k);
			}
			gsl_matrix_set(XtX, i, j, sum);
		}
	}

	// I = XtX^-1
	gsl_linalg_LU_decomp(XtX, p, &s);
	gsl_linalg_LU_invert(XtX, p, I);
	
	// C = I*Xt
	for(i=0; i<m; i++)
	{
		for(j=0; j<n; j++)
		{
			double sum = 0.0;
			for(k=0; k<m; k++)
			{
				sum += gsl_matrix_get(I, i, k) * gsl_matrix_get(Xt, k, j);
			}
			gsl_matrix_set(&C.matrix, i, j, sum);
		}
	}
	
	// a = C*y
	for(i=0; i<m; i++)
	{
		double sum = 0.0;
		for(j=0; j<n; j++)
		{
			sum += gsl_matrix_get(&C.matrix, i, j) * y_data[j];
		}
		coeff[i] = sum;
	}
	
	// free
	gsl_matrix_free(Xt);
	gsl_matrix_free(XtX);
	gsl_matrix_free(I);
	gsl_permutation_free(p);
}
