#include <psquarequantile.h>

#include <cfloat>
#include <cassert>
#include <iostream>

PSquareQuantile::PSquareQuantile(double p)
{
	_p[0] = p/2.0;
	_p[1] = p;
	_p[2] = (1+p)/2.0;
		
	reset();
}

void PSquareQuantile::advance(double value)
{
	unsigned int k;
	
	// ersten 5 Werte aufsteigend sortieren
	if(_init)
	{
		unsigned int insert=0;
		for(; insert<5; insert++)
		{
			// first larger entry
			if(value>=_q[insert]) continue;

			// swap
			for(unsigned int j=4; j>insert; j--)
				_q[j] = _q[j-1];
			
			break;
		}
		_q[insert] = value;
	
		_init--;
		return;
	}
	
	// insertion cell
	k = 1;
	
	// new minimum
	if(value < _q[0]) _q[0] = value;
	else if(_q[1] <= value && value < _q[2]) k = 2;
	else if(_q[2] <= value && value < _q[3]) k = 3;
	else if(_q[3] <= value && value < _q[4]) k = 4;
	// new maximum
	else if(value > _q[4]) { _q[4] = value; k = 4; }
		
	for(unsigned int i=0; i<5; i++)
	{
		if(i>=k) _n[i]++;
		
		_nn[i] += _dn[i];
	}
	
	for(unsigned int i=1; i<4; i++)
	{
		double d = _nn[i] - _n[i];
			
		if(((d >= 1.0) && (_n[i+1] - _n[i] > 1.0))
			|| ((d <= -1.0) && (_n[i-1] - _n[i] < -1.0)))
		{
			// signum
			d = d < 0.0 ? -1.0 : 1.0;
		
			// parabolic prediction
			double t1 = (_n[i] - _n[i-1] + d) * ((_q[i+1] - _q[i]) / (_n[i+1] - _n[i]));
			double t2 = (_n[i+1] - _n[i] - d) * ((_q[i] - _q[i-1]) / (_n[i] - _n[i-1]));
			double qi = _q[i] + (d / (_n[i+1] - _n[i-1])) * (t1 + t2);

			//std::cerr << "q" << i << " (parabolic): " << qi << std::endl;
			
			// check parabolic prediction
			if(_q[i-1] < qi && qi < _q[i+1])
				_q[i] = qi;
			else
			{
				// linear prediction
				_q[i] = _q[i] + d * (_q[i+(int)d] - _q[i]) / (_n[i+(int)d] - _n[i]);
			
				//std::cerr << "q" << i << " (linear): " << _q[i] << std::endl;
			}
			_n[i] = _n[i] + d;
		}	
	}
}

void PSquareQuantile::reset()
{
	for(unsigned int i=0; i<5; i++)
	{
		_q[i] = DBL_MAX;
		_n[i] = i+1;
	}

	_nn[0] = 1;
	_nn[1] = 1 + 2 * _p[1];
	_nn[2] = 1 + 4 * _p[1];
	_nn[3] = 3 + 2 * _p[1];
	_nn[4] = 5;

	_dn[0] = 0;
	_dn[1] = _p[1] / 2.0;
	_dn[2] = _p[1];
	_dn[3] = (1 + _p[1]) / 2.0;
	_dn[4] = 1;

	_init = 5;
}
