#include <variable.h>
#include <spectralmean.h>
#include <independentmean.h>
#include <spectralquantile.h>
#include <initialtransient.h>
#include <mathematics.h>
#include <macros.h>

#include <cassert>
#include <iostream>


Variable::Variable(const Analysis::VariableInfo& info)
{
	advanceInternal=&Variable::advanceEstimation;
	
	_state = ESTIMATION;
	_est = NULL;
	_itd = NULL;
	
	if(info.type == Analysis::VariableInfo::Mean)
	{
		if(info.independent)
			_est = new IndependentMean(info.confidence);

		else
			_est = new SpectralMean(info.confidence);

	}
	
	if(info.type == Analysis::VariableInfo::Quantile)
	{
		_est = new SpectralQuantile(info.confidence, info.quantile);
	}

	
	if(info.initialtransient)
	{
		advanceInternal=&Variable::advanceTransient;
		
		_state = TRANS_DETECT;
		_itd = new InitialTransient();
	}

	assert(_est);
	assert(_state != TRANS_DETECT || _itd);
}


Variable::~Variable()
{
	if(_est)
		delete _est;

	if(_itd);
		delete _itd;
}


bool Variable::advanceTransient(double value, Analysis::Result& result)
{
	assert(_state==TRANS_DETECT);

	result.values++;
	
	switch(_itd->advance(value))
	{		
	case InitialTransient::FINISHED:
		
		result.discarded = _itd->discarded();
		result.values = result.values - result.discarded;
		
		_state=ESTIMATION;
		advanceInternal=&Variable::advanceEstimation;
		
		//
		// Stabile Werte aus Test in Auswertung uebernehmen
		//
		return (*_itd >> _est);
		
	case InitialTransient::FAILED:
	
		_WARNING("Failed to detect initial transient! Estimation will be skipped.");
		
		_state=FINISHED;
		advanceInternal=&Variable::advanceFinished;
		break;

	default: 
		break;
	}

	return false;
}

bool Variable::advanceEstimation(double value, Analysis::Result& result)
{
	assert(_state==ESTIMATION);
		
	// update
	result.values++;
	
	// checkpoint
	if(_est->advance(value))
	{
		(*_est) >> result;
	
		return true;
	}
	
	return false;
}

bool Variable::advanceFinished(double value, Analysis::Result& result)
{
	assert(_state==FINISHED);

	return false;
}

