#ifndef __CINSIMENV_H__
#define __CINSIMENV_H__

#include <environment.h>

//! Laufzeitumgebung fuer Simulation.
/*!
 * Die Laufzeitumgebung fuer eine Simulation muss einen Generator
 * fuer Zufallszahlen und eine Instanz der stochastischen Auswertung
 * bereitstellen. Weiterhin werden die Plugins identifiziert und
 * bereitgestellt. 
 * 
 * Fuer verteilte Simulation mit MPI wird die MPI-Umgebung initialisiert
 * und verschiedene Streams von Zufallszahlen fuer die verteilten
 * Instanzen der Simulation vergeben.
 * 
 * @see Environment
 * @see PluginManager
 * @see Analysis
 * @see PRNG
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class CINSimEnv : public Environment
{
	
protected:	

	//! Implementierung der abstrakten Klasse Environment.
	/*!
	 * Die Stochastische Auswertung und der Zufallszahlengenerator 
	 * werden initialsiert, Plugins werden geladen und der Nice-Level
	 * fuer die Ausfuehrung gesetzt.
	 * 
	 * Bei verteilten Simulationen mit MPI wird die MPI-Umgebung
	 * initialisiert und verschiedene Streams von Zufallszahlen
	 * fuer die einzelnen Instanzen der Simulation vergeben.
	 *
	 * @see Environment
	 */
	virtual bool initialize(int argc, char** argv);

	//! Implementierung der abstrakten Klasse Environment.
	/*!
	 * Bei verteilten Simulationen mit MPI wird die MPI-Umgebung
	 * zurueckgesetzt.
	 * 
	 * @see Environment
	 */
	virtual bool finalize();	
	
	/*! Implementierung der abstrakten Klasse Environment.
	 *
	 * @see Environment.
	 */
	virtual CmdParser* createCmdParser() const;
};

#endif
