#ifndef __RAND48_H__
#define __RAND48_H__

#include <environment/prng.h>

//! Generator fuer Pseudo-Zufallszahlen aus der C-Bibliothek.
/*! 
 * Diese Klasse implementiert die abstrakte Klasse PRNG mit
 * Methodem der C-Bibliothek (Rand48). Implementierung ist nicht
 * fuer verteilte Simulationen geeignet.
 *
 * @see PRNG
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Rand48 : public PRNG
{
public:

	//! Konstruktor.
	/*!
	 * Der Aufruf des Konstruktors erzeugt einen Generator basierend auf
	 * Methoden der C-Library (SVID).
	 */
	Rand48(); 

	//! Destruktor.
	/*!
	 * Die interne Generator-Struktur wird freigegeben.
	 */
	~Rand48() {}
	
	//! Implementierung der Basisklasse PRNG.
	/*!
	 * @see PRNG::uniformInt()
	 */
	unsigned int uniformInt(unsigned int start, unsigned int end);
	
protected:
	
	//! Implementierung der Basisklasse PRNG.
	/*!
	 * @see PRNG::uniqueStreamId()
	 */
	void uniqueStreamId(unsigned int uid);
		
private:
	
	//! Zustandsbeschreibung des Generators.
	unsigned short state[3];	
};

#endif
