#include <cppunit/TextTestRunner.h>

#include <prng/test_cmrg.h>
#include <prng/test_mtwister.h>
#include <prng/test_rand48.h>

#include <analysis/test_mathematics.h>
#include <analysis/test_psquarequantile.h>
#include <analysis/test_estimation.h>

#include <cassert>

/*!
 * Unit-Tests fuer Simulationsumgebung.
 *
 * Einige Tests fuer Zufallszahlen und statistische Auswertung.
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 * @return 0 bei Erfolg, != 0 sonst
 */
int main(int argc, char** argv)
{
	CppUnit::TextTestRunner runner;
	
	// Zufallszahlen
	runner.addTest( Test_PRNG::suite() );
	runner.addTest( Test_CMRG::suite() );
	runner.addTest( Test_MTwister::suite() );
	runner.addTest( Test_Rand48::suite() );
	
	// Hilfsmittel
	runner.addTest( Test_Mathematics::suite() );
	runner.addTest( Test_PSquareQuantile::suite() );
	
	// Abschaetzungen
	runner.addTest( Test_IndependentMean::suite() );
	runner.addTest( Test_SpectralMean::suite() );
	runner.addTest( Test_SpectralQuantile::suite() );

	return !runner.run("", false);
}
