#include <packet.h>
#include <buffer.h>
#include <flit.h>
#include <utility/bitmask.h>
#include <configuration.h>
#include <macros.h>

#include <cassert>

unsigned int Packet::_packetNumber = 0;
unsigned int Packet::_packetCount = 0;

Packet::Packet()
{
	// aufsteigende Paketnummern
	number=_packetNumber++;

	_packetCount++;
	
	refcount=0;	
	ownedflits=0;
}

Packet::Packet(const Packet& packet)
{
	// flache Kopie
	(*this) = packet;
		
	_packetCount++;
	
	refcount=0;
	ownedflits=0;
}

Packet::~Packet()
{
	// Annahme: Referenzzaehler initial oder 1
	assert(refcount <= 1);
	// Annahme: Anzahl eigener Flits uebersteigt nicht Laenge
	assert(ownedflits <= size());
	
	// Annahme: Globale Anzahl von Paketen nicht 0
	assert(_packetCount);
	
	for(unsigned int i=0; i<ownedflits; i++)
	{		
		Flit* fptr = (*this)[i];

		// Annahme: Eigenes Flit existiert
		assert(fptr && fptr->Owner() == this);
		
		delete fptr;
		(*this)[i] = NULL;
	}

	_packetCount--;
}

bool Packet::resize(unsigned int newsize)
{
	newsize = newsize<1 ? 1 : newsize;
	
	// Annahme: Aufruf nur durch Konstruktor, daher Groesse 0
	assert(size() == 0);
	
	unsigned int offset = 0;
	while(newsize > offset)
	{
		push_back(this->createFlit(offset,newsize-1));
		offset++;
	}

	// Alle Flits gehoeren zu diesem Paket
	ownedflits = newsize;
	
	return true;
}

std::string Packet::toString() const 
{
	std::stringstream str;

	str << "Packet " << number << " -> "
		<< "source: " << sourcenr 
		<< ", size: " << size()
		<< ", created: " << createTick;

	return str.str();
}

