#include <packetforwarding.h>
#include <macros.h>
#include <utility/xmlhelper.h>
#include <routing.h>
#include <switching.h>
#include <buffer.h>
#include <bufferqueue.h>
#include <wirematrixentry.h>
#include <wirematrixiterator.h>
#include <packet.h>

#include <simulation.h>

#include <cassert>

PacketForwarding::~PacketForwarding()
{
	if(_routing)
		delete _routing;

	if(_switching)
		delete _switching;

	if(_forwarding)
		delete _forwarding;
}


bool PacketForwarding::init(const DOMElement* scope)
{
	if(scope) setScope(scope);
	
	if(!Scope())
		_ERROR("Could not initialize coroutine, no XML spezified!"); 

	_routing = (Routing*)FactoryElement::Factory::self().
			create(conv(Scope()->getAttribute(conv("routingType"))));

	if(!_routing) 
		_ERROR("Unknown routing method ("  
				<< conv(Scope()->getAttribute(conv("routingType"))) << ")!");
	
	_switching = (Switching*)FactoryElement::Factory::self().
			create(conv(Scope()->getAttribute(conv("switchingType"))));
	
	// Setze wenn ntig ein Switching-Flag
	if(conv(Scope()->getAttribute(conv("switchingType")))[0]=='P')
		isPCT = true;
	if(conv(Scope()->getAttribute(conv("switchingType")))[0]=='V')
		isVCT = true;

	if(!_switching)                
		_ERROR("Unknown switching method ("  
			<< conv(Scope()->getAttribute(conv("switchingType"))) << ")!");

	// Abgeleitet Klassen initialisieren
	_forwarding = initForwarding();

	if(!_forwarding)
		_ERROR("Expected instance of ForwardingModifier, found NULL!");
		
	return _routing->init();
}


void ForwardingModifier::modify(WireMatrixIterator* it)
{
	WireMatrixEntry& entry = it->value();
	Buffer* next = (Buffer*)entry.getOutWire()->To();
	Buffer* pre = (Buffer*)entry.getInWire()->From();
			
	// Vermittelbares Flit fuer Ausgang
	const Flit* online = entry.getInWire()->getFlit(next);
	
	// Annahme: Ein vermittelbares Flit fuer Ausgang liegt vor
	assert(online);

	entry.set(IO_FIT, (next->BQ->freeSpace() && 
			(next->BQ->estimatedSpace() >= online->neededSpace())
			// IO_FIT nur erlauben, wenn nicht (pre und next.infinite).
			&& (pre->BQ->infinite || !next->BQ->infinite)
			));

}

