#include <test_triggernode.h>
#include <triggernode.h>

#include <buffer.h>
#include <iostream>

CPPUNIT_TEST_SUITE_REGISTRATION( Test_TriggerNode );

void Test_TriggerNode::setUp()
{
	tnode0 = new Buffer;
	tnode1 = new Buffer;
	tnode2 = new Buffer;

	notsigned = new SignedTNodeList;
	sent = new SignedTNodeList(TriggerNode::PULLED);
}


void Test_TriggerNode::init_check()
{
	// noch keine Nachfolgeknoten 
	CPPUNIT_ASSERT( tnode0->children().size()==0 );
	CPPUNIT_ASSERT( !tnode0->hasChildren() );
	
	// nicht signiert
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::PUSHED) );
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::PULLED) );
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::ACTIVE) );

	// Listen leer
	CPPUNIT_ASSERT( notsigned->empty() );
	CPPUNIT_ASSERT( sent->empty() );
}


void Test_TriggerNode::construction_check()
{
	// Nachfolgeknoten hinzufuegen
	tnode0->addChild(tnode1);
	tnode0->addChild(tnode2);

	// Hinzfuegen der Knoten erfolgreich
	CPPUNIT_ASSERT( tnode0->hasChildren() );
	CPPUNIT_ASSERT( tnode0->children().size()==2 );

	// Reihenfolge egal, aber beide Knoten vorhanden 
	CPPUNIT_ASSERT( tnode0->children()[0]!=tnode1 || tnode0->children()[1]==tnode2 );
	CPPUNIT_ASSERT( tnode0->children()[0]!=tnode2 || tnode0->children()[1]==tnode1 );	

	// Hinzufuegen ohne Einfluss auf Signaturen
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::PUSHED) );
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::PULLED) );
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::ACTIVE) );	

	// kein Einfluss auf Signaturen hinzugefuegter Knoten
	CPPUNIT_ASSERT( !tnode1->test(TriggerNode::PUSHED) );
	CPPUNIT_ASSERT( !tnode1->test(TriggerNode::PULLED) );
	CPPUNIT_ASSERT( !tnode1->test(TriggerNode::ACTIVE) );		
}


void Test_TriggerNode::signing_check()
{
	// Liste unsignierter Knoten
	notsigned->push_front(tnode0);
	notsigned->push_back(tnode0);
	notsigned->insert(notsigned->begin(), tnode0);
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::PUSHED) );
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::PULLED) );
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::ACTIVE) );
	CPPUNIT_ASSERT( notsigned->size()==3);

	// Liste signierter Knoten
	sent->push_front(tnode0);
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::PUSHED) );
	CPPUNIT_ASSERT( tnode0->test(TriggerNode::PULLED) );
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::ACTIVE) );

	sent->push_back(tnode1);
	CPPUNIT_ASSERT( !tnode1->test(TriggerNode::PUSHED) );
	CPPUNIT_ASSERT( tnode1->test(TriggerNode::PULLED) );
	CPPUNIT_ASSERT( !tnode1->test(TriggerNode::ACTIVE) );	
	
	sent->insert(sent->begin(), tnode2);
	CPPUNIT_ASSERT( !tnode2->test(TriggerNode::PUSHED) );
	CPPUNIT_ASSERT( tnode2->test(TriggerNode::PULLED) );
	CPPUNIT_ASSERT( !tnode2->test(TriggerNode::ACTIVE) );

	CPPUNIT_ASSERT( sent->size()==3);
	
	// signierte Knoten werden ignoriert
	sent->push_front(tnode0);
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::PUSHED) );
	CPPUNIT_ASSERT( tnode0->test(TriggerNode::PULLED) );
	CPPUNIT_ASSERT( !tnode0->test(TriggerNode::ACTIVE) );
	
	CPPUNIT_ASSERT( sent->size()==3);
}


void Test_TriggerNode::tearDown()
{
	if (tnode0)
		delete tnode0;
	if (tnode1)
		delete tnode1;
	if (tnode2)
		delete tnode2;
	if (notsigned)
		delete notsigned;
	if (sent)
		delete sent;
}



