#ifndef __GEOMETRIC_H__
#define __GEOMETRIC_H__

#include <traffic.h>

//! Zufaelliger Verkehr anhand geometrischer Verteilung. 
/*!
 * Diese zeitliche Verteilung von Paketen basiert auf der geometrischen 
 * Verteilung. Es wird allerdings noch die Groesse eines Paketes einbezogen,
 * was sich bei der Generierung von Paketen mit mehr als einem Flit auswirkt.
 *
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class GeometricDistribution : public TrafficDistribution 
{
public:
	
    //! Konstruktor.
	/*!
	 * Der Konstruktor initialisiert lediglich die Attribute der Basisklassen.
	 *
	 * @see TrafficDistribution
	 * @see XMLElement
	 * 
	 * @param psize Groesse der zu generierenden Pakete.
	 * @param scope Gueltigkeitsbereich innerhalb des XML-Dokumentes.
	 */
    GeometricDistribution(unsigned int psize=1, const DOMElement* scope=NULL)
		: TrafficDistribution(psize,scope) {}
    
	//! Implementierung der Basisklasse XMLElement.
	/*!
	 * Die Verteilung wird anhand der relavanten Attribute aus dem
	 * Gueltigkeitsbereich innerhalb des XML-Dokumentes initialisiert.
	 *
	 * @see XMLElement.
	 * 
	 * @param scope Der Gueltigkeitsbereich.
	 * @return true.
	 */
	bool init(const DOMElement* scope = NULL);
	
	//! Implementierung der Basisklasse TrafficDistribution.
	/*!
	 * Der Parameter fuer die geometrische Vertielung bestimmt die Wahrscheinlichkeit
	 * mit der ein Paket generiert werden kann, Zusaetzlich wird die Laenge der zu 
	 * generierenden Pakete beruecksichtigt.
	 * 
	 * @see TrafficDistribution
	 *
	 * @param tick Der aktuelle Takt.
	 * @return true, falls laut Verteilung Paket generiert werden kann.
	 */
    bool ifNowGenerate(unsigned int tick);
    
	//! Implementierung der Basisklasse TrafficDistribution.
	/*!
	 * Der interne Zustand dieser Verteilung wird nur durch die (globale)
	 * Sequenz von Zufallszahlen der Simulation bestimmt.
	 */
	void reset() {}
	
	//! Destruktor.
	~GeometricDistribution() {};
    
private:
	
	//! Parameter fuer geometrische Verteilung.
    double load;
};

#endif

