#include <bitmask.h>
#include <assert.h>
#include <iostream>

#define MAX(a,b) (a) > (b) ? (a) : (b)

void Bitmask::set(unsigned int bit) 
{	
	farray[FIELD_INDEX(bit)] |= (1 << BIT_INDEX(bit));
}

void Bitmask::reset(unsigned int bit)
{
	unsigned int ftmp = FIELD_INDEX(bit);
	
	if(ftmp<farray.size)
		farray(ftmp) &= farray(ftmp) ^ (1 << BIT_INDEX(bit));
}

bool Bitmask::test(unsigned int bit) const
{
	if(FIELD_INDEX(bit)<farray.size)
		return farray.read(FIELD_INDEX(bit)) & (1 << BIT_INDEX(bit));
	return false;
}

bool Bitmask::empty() const
{
	for(unsigned int i=0; i<farray.size; i++)
		if(farray.read(i)) return false;
	return true;
}

void Bitmask::clear()
{
	farray.resize(1);
	farray[0]=0;
}

// Nichtleerer Schnitt
bool Bitmask::operator<=(const Bitmask& op) const
{
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size, op.farray.size);
	for(; field<eofa; field++)
		if(this->farray.read(field) & op.farray.read(field))
			return true;
		
	return false;
}

// Gleichheit
bool Bitmask::operator==(const Bitmask& op) const
{
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size, op.farray.size);
	for(; field<eofa; field++)
		if(this->farray.read(field) != op.farray.read(field))
			return false;
	
	return true;
}

// Bitweiser Schnitt (UND)
Bitmask Bitmask::operator&(const Bitmask& op) const
{
	Bitmask result;
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size, op.farray.size);
	result.farray.resize(eofa);
	for(; field<eofa; field++)
		result.farray(field)=this->farray.read(field) & op.farray.read(field); 
	
	return result;
}

// Bitweise Vereinigung (ODER)
Bitmask Bitmask::operator|(const Bitmask& op) const
{
	Bitmask result;
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size, op.farray.size);
	result.farray.resize(eofa);
	for(; field<eofa; field++)
		result.farray(field)=this->farray.read(field) | op.farray.read(field); 
	
	return result;
}

// Bitweise Exklusiv-Vereinigung (XOR)
Bitmask Bitmask::operator^(const Bitmask& op) const
{
	Bitmask result;
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size, op.farray.size);
	result.farray.resize(eofa);
	for(; field<eofa; field++)
		result.farray(field)=this->farray.read(field) ^ op.farray.read(field); 
		
	return result;
}

// Bitweise Differenz zweier Bitmasken
Bitmask Bitmask::operator-(const Bitmask& op) const
{
	Bitmask result;
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size,op.farray.size);
	result.farray.resize(eofa);
	for(; field<eofa; field++){
		unsigned int tmp = this->farray.read(field); 	
		result.farray(field) = tmp ^ (tmp & op.farray.read(field));
	}
	return result;
}
	
// Bitweiser Schnitt mit Zuweisung
Bitmask& Bitmask::operator&=(const Bitmask& op)
{	
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size,op.farray.size);
	this->farray.resize(eofa);	
	for(; field<eofa; field++)
		this->farray(field) = this->farray.read(field) & op.farray.read(field);
		
	return (*this);
}

// Bitweise Vereinigung mit Zuweisung
Bitmask& Bitmask::operator|=(const Bitmask& op) 
{
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size,op.farray.size);
	this->farray.resize(eofa);	
	for(; field<eofa; field++)
		this->farray(field) = this->farray.read(field) | op.farray.read(field);
		
	return (*this);
}

// Bitweise Exklusiv-Vereinigung mit Zuweisung
Bitmask& Bitmask::operator^=(const Bitmask& op) 
{
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size,op.farray.size);
	this->farray.resize(eofa);	
	for(; field<eofa; field++)
		this->farray(field) = this->farray.read(field) ^ op.farray.read(field);
		
	return (*this);
}
	
// Bitweise Differenz mit Zuweisung
Bitmask& Bitmask::operator-=(const Bitmask& op) 
{
	unsigned int field=0;
	unsigned int eofa=MAX(this->farray.size,op.farray.size);
	this->farray.resize(eofa);	
	for(; field<eofa; field++){
		unsigned int tmp = this->farray.read(field);
		this->farray(field) = tmp ^ (tmp & op.farray.read(field));
	}
		
	return (*this);
}
		
// Konstruktur fuer Bitfeld
Bitmask::FieldArray::FieldArray(unsigned int fields)
{
	inner_array = new unsigned int[fields];
	for(unsigned int i=0; i<fields; i++)
		*(inner_array+i)=0;
			
	size=fields;
}
	
// Dimensionsaenderung
void Bitmask::FieldArray::resize(unsigned int newsize)
{	
	// mindestens ein Bitfeld
	newsize=newsize?newsize:1;
	
	// keine Veraenderung
	if(newsize==size)
		return;
			
	unsigned int* new_array = new unsigned int[newsize];
	unsigned int offset=0;
	
	while(size && newsize)
	{
		*(new_array+offset) = *(inner_array+offset);
		newsize--;
		size--;
		offset++;
	}
	while(newsize)
	{
		*(new_array+offset) = 0;
		newsize--;
		offset++;
	}
	delete[] inner_array;
	inner_array=new_array;
	size=offset;
}
		
// Zuweisungs-Operator.
Bitmask::FieldArray& Bitmask::FieldArray::operator=(const FieldArray& op)
{
	unsigned int newsize=op.size;
	unsigned int offset=0;

	if(size != op.size)
	{
		delete[] inner_array;
		inner_array=new unsigned int[newsize];
	}
	
	while(newsize)
	{
		*(inner_array+offset) = *(op.inner_array+offset);
		newsize--;
		offset++;
	}
	
	size=offset;
	return (*this);
}

