#ifndef __TEST_BITMASK_H__
#define __TEST_BITMASK_H__

#include <cppunit/extensions/HelperMacros.h>
#include <vector>

#include <bitmask.h>

using namespace CppUnit;

/*!
 * Unit-Test(s) fuer Klasse Bitmask.
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Test_Bitmask : public TestFixture
{	
CPPUNIT_TEST_SUITE( Test_Bitmask );
CPPUNIT_TEST( init_check );
CPPUNIT_TEST( size_check );
CPPUNIT_TEST( and_check );
CPPUNIT_TEST( or_check );
CPPUNIT_TEST( xor_check );	
CPPUNIT_TEST( diff_check );
CPPUNIT_TEST( assignment_check );	
CPPUNIT_TEST( relation_check );
CPPUNIT_TEST_SUITE_END();


public:

	/*! 
	 * Implementierung der Klasse TestFixture.
	 *
	 * @see TestFixture
	 */
	void setUp();
	
	/*! 
	 * Implementierung der Klasse TestFixture.
	 *
	 * @see TestFixture
	 */	
	void tearDown();
	
	//! Test auf initial leere Bitmaske.
	/*! 
	 * Fuer eine neue Bitmask soll stets Bitmask::empty() gelten,
	 * nach dem Setzen des ersten Bits nicht mehr.
	 */
	void init_check();

	//! Test auf beliebige Anzahl von Bits.
	/*!
	 * Die Anzahl der Bits, die eine Bitmaske verwalten kann, soll
	 * beliebieg sein. Dazu muss jeweils dynamisch Speicher fuer neue
	 * Bits alloziiert werden. Setzen, Loeschen und Testen der Bits
	 * sollte davon unbeeinflusst bleiben.
	 */
	void size_check();

	//! Test der AND-Operatoren.
	void and_check();

	//! Test der OR-Operatoren.
	void or_check();

	//! Test der XOR-Operatoren.
	void xor_check();

	//! Test der Differenz-Operatoren.
	void diff_check();
	
	//! Test der Zuweisungs-Operatoren und des Copy-Konstruktors.
	/*!
	 * Es sollen bei Zuweisung und Anwendng des Copy-Konstruktors
	 * tiefe Kopien mit korrekt gesetzten Bits entstehen.
	 */
	void assignment_check();		
	
	//! Test der Vergleichsoperatoren.
	void relation_check();

private:

	//! Bitmasken fuer Tests.
	Bitmask *mask0, *mask1, *mask2, *mask3;

	//! Testbits.
	std::vector<unsigned int> bits;
};


#endif
