#include <utility/xmlhelper.h>
#include <macros.h>
#include <iostream>

#include <cassert>

//#include <xercesc/dom/DOM.hpp>
//#include <xercesc/util/XMLString.hpp>
#include <xercesc/framework/psvi/XSElementDeclaration.hpp>
//#include <xercesc/framework/psvi/XSTypeDefinition.hpp>

using namespace std;

// Vergleichsmethode fuer Element-Filter. 
#if _XERCES_VERSION >= 30000
  DOMNodeFilter::FilterAction ElementFilter::acceptNode(const DOMNode* node) const {
#else
  short ElementFilter::acceptNode(const DOMNode* node) const {
#endif
	if(node->getNodeType() == DOMNode::ELEMENT_NODE)
	{
		if(!tagname || filter(node->getNodeName(), tagname))
		{
			return DOMNodeFilter::FILTER_ACCEPT;
		}
	}
	return DOMNodeFilter::FILTER_SKIP;
}


// (Hilfs)Methode zum Stringverglich 
bool equals(const XMLCh* str1, const char* str2)
{
	return XMLString::equals(str1,XMLString::transcode(str2));
}


// (Hilfs)Methode zum Stringverglich 
bool equalsIgnoreCase(const XMLCh* str1, const char* str2)
{
	XMLCh* cstr1 = XMLString::replicate(str1);
	XMLString::lowerCase(cstr1);
	
	XMLCh* cstr2 = XMLString::transcode(str2);
	XMLString::lowerCase(cstr2);
	
	return XMLString::equals(cstr1,cstr2);
}


// (Hilfs)Methode zum Stringvergleich
bool endsWith(const XMLCh* str1, const char* str2)
{
	return XMLString::endsWith(str1,XMLString::transcode(str2));
}


// (Hilfs)Methode zum Stringvergleich
bool startsWith(const XMLCh* str1, const char* str2)
{
	return XMLString::startsWith(str1,XMLString::transcode(str2));
}


// Konvertierungsfunktion C-String -> Xerces-String
const XMLCh* conv(const char* str)
{
	return XMLString::transcode(str);
}


// Konvertierungsfunktion Xerces-String -> C-String 
char* conv(const XMLCh* str)
{
	return XMLString::transcode(str);
}



XMLAttribute parseAttribute(const DOMElement* elem, const char* name)
{
	assert(elem && name);
	
	DOMAttr* attribute = elem->getAttributeNode(conv(name));

	if(!attribute)
		_ERROR("No Attribute '" << name << "' for element " 
			<< StrX(elem->getTagName()) << " found!"
			<< " Element-ID is " << StrX(elem->getAttribute(conv("id"))));
			
	return XMLAttribute(attribute);
}


vector<DOMElement*> getChildElementsByName(DOMElement* element, const char* tagname, bool deep, bool (*filter)(const XMLCh*,const char*))
{	
	vector<DOMElement*> found;

	ElementFilter* ef = new ElementFilter(tagname, filter);	
		
	DOMTreeWalker* luke = element->getOwnerDocument()->
			createTreeWalker(element, DOMNodeFilter::SHOW_ELEMENT, ef, deep);

	DOMElement* child = (DOMElement*)luke->firstChild();

	while(child)
	{
		found.push_back(child);
		child = (DOMElement*)luke->nextSibling();
	}
	return found;
}


DOMElement* getFirstChildByName(const DOMElement* element, const char* tagname, bool (*filter)(const XMLCh*,const char*))
{
	ElementFilter* ef = new ElementFilter(tagname, filter);	
		
	DOMTreeWalker* luke = element->getOwnerDocument()->
			createTreeWalker((DOMElement*)element, DOMNodeFilter::SHOW_ELEMENT, ef, false);

	return (DOMElement*)luke->firstChild();
}


DOMElement* getNextSiblingByName(DOMElement* element, const char* tagname, bool (*filter)(const XMLCh*,const char*))
{
	ElementFilter* ef = new ElementFilter(tagname, filter);	
	
	DOMTreeWalker* luke = element->getOwnerDocument()->
			createTreeWalker((DOMElement*)element->getParentNode(), DOMNodeFilter::SHOW_ELEMENT, ef, false);

	luke->setCurrentNode(element);
	
	DOMElement* sibling = (DOMElement*)luke->nextSibling();

	return sibling;
}


DOMTreeWalker* createElementWalker(DOMElement* root, const char* tagname, bool deep, bool (*filter)(const XMLCh*,const char*))
{
	ElementFilter* ef = new ElementFilter(tagname, filter);	
	
	return root->getOwnerDocument()->
			createTreeWalker(root, DOMNodeFilter::SHOW_ELEMENT, ef, deep);
}

DOMNodeIterator* createNodeIterator(DOMElement* root, char* tagname, bool deep, bool (*filter)(const XMLCh*,const char*))
{
	ElementFilter* ef = new ElementFilter(tagname, filter);	
	
	return root->getOwnerDocument()->
			createNodeIterator(root, DOMNodeFilter::SHOW_ELEMENT, ef, deep);
}

int mergeElements(const DOMElement* dom1, DOMElement* dom2) 
{
	if(!dom1 || !dom2) return 0;
	
	DOMNamedNodeMap* attributes = dom1->getAttributes(); 
	
	for (unsigned int i=0; i<attributes->getLength(); i++)
		if(!addAttribute(dom2,(DOMAttr*)attributes->item(i))) return 0;
	
	return 1;
}

int addAttribute(DOMElement* elem, DOMAttr* attr)
{
	if(!elem || !attr) 
	{
		_WARNING("Invalid argument(s)!");
		return 0;
	}
	
	if (!elem->hasAttribute (attr->getName())) 
	{
		try 
		{
			elem->setAttributeNode((DOMAttr*)attr->cloneNode(true));
		} 
		catch (DOMException ex) 
		{
			_WARNING("Exception caught! Code: " << ex.code);
			return 0;
		}
		//std::cerr << "...." << StrX(attr->getName()) << " uebernommen!" << std::endl;
		return 2;
	}
	//std::cerr << "...." << StrX(attr->getName()) << " vorhanden!" << std::endl;
	return 1;
}


// original: bool isDerivedFrom(const SchemaInformation* sinfo, const DOMElement* element, const char* type)
bool isDerivedFrom(const SchemaInformation* sinfo, const DOMElement* element, const char* type)
{
	if(!sinfo || !element || !type)
	{
		_WARNING("Invalid argument(s)!");
		return false;
	}

//  	xercesc_3_0::XSElementDeclaration* ed;// = sinfo->getValue(conv(element->getTagName()));
//  	*ed = sinfo->getValue(conv(element->getTagName()));
	
 	/*original:*/ const XSElementDeclaration* ed = sinfo->getValue(conv(element->getTagName()));
// 	const XSElementDeclaration* ed;
// 	int groesse = sinfo->size();
// 	string teststring = conv(element->getTagName());
// 	//sinfo->getValue(conv(element->getTagName()));
// 	sinfo->getValue(teststring);
	
	if(ed)
	{
		XSTypeDefinition *td = ed->getTypeDefinition();
		while(td && !equals(td->getName(), "anyType"))
		{
			if(equals(td->getName(), type))
				return true;
		
			td = td->getBaseType();
		}
	}
	
	else _WARNING("No type information for element " << StrX(element->getTagName())
			<< " found!"); 
		
	return false;
}
