#include <wirematrix.h>
#include <wirematrixiterator.h>
#include <wire.h>
#include <macros.h>

WireMatrixIterator* WireMatrix::iterateOutWires(Wire* inwire)
{
	int row_index = inwire->getInIndex();

	if(row_index >= getYSize())
		_ERROR("Trying to iterate outwires for inwire out of range! " 
			<< "Index=" << row_index << ", " << "Size=" << getYSize());
	
	// the first time...
	if(!ow_iterators[row_index])
		ow_iterators[row_index] = new OutWireIterator(row_begin(row_index), row_end(row_index), iw_iterators);

	ow_iterators[row_index]->reset();
	return ow_iterators[row_index];
}


WireMatrixIterator* WireMatrix::iterateInWires(Wire* outwire)
{
	int col_index = outwire->getOutIndex();

	if(col_index >= getXSize())
		_ERROR("Trying to iterate inwires for outwires out of range! " 
			<< "Index=" << col_index << ", " << "Size=" << getXSize());
	
	// the first time...
	if(!iw_iterators[col_index])
		iw_iterators[col_index] = new InWireIterator(col_begin(col_index), col_end(col_index), ow_iterators);
	
	iw_iterators[col_index]->reset();
	return iw_iterators[col_index];
}

void WireMatrix::newInWire()
{
	std::vector<WireMatrixEntry> row;	
	PacketInfo p_info;
	p_info.remain=new Bitmask();
	
	for(unsigned int i=0; i<outwires.size(); i++)
	{
		WireMatrixEntry entry(this, inwires.size()-1, i);

		row.push_back(entry);
	}

	// new dimension for matrix
	m_Matrix->push_back(row);	
	m_nYSize = inwires.size();

	// place for storing iterators
	ow_iterators.push_back(NULL);	
	
	// Platz fuer Informationen zu anliegenden Paket
	m_packets.push_back(p_info);
}


void WireMatrix::newOutWire()
{	
	for(unsigned int i=0; i<inwires.size(); i++)
	{
		WireMatrixEntry entry(this, i, outwires.size()-1);

		m_Matrix->at(i).push_back(entry);
	}

	// new dimension for matrix
	m_nXSize = outwires.size();

	// place for storing iterators
	iw_iterators.push_back(NULL);
}


