#include <wirematrixentry.h>
#include <wirematrix.h>
#include <wire.h>
#include <utility/bitmask.h>
#include <bufferqueue.h>
#include <registercontainer.h>

#include <assert.h>

Wire* WireMatrixEntry::getOutWire() const
{
	// Annahme: keine Bereichueberschreitung
	assert(_wm && _wm->outwires.size() > ow_index);
	
	return _wm->outwires[ow_index]; 
}

Wire* WireMatrixEntry::getInWire() const
{
	// Annahme: keine Bereichueberschreitung
	assert(_wm && _wm->inwires.size() > iw_index);
	
	return _wm->inwires[iw_index]; 
}

const Packet* WireMatrixEntry::currentPacket() const
{
	// Annahme: keine Bereichueberschreitung
	assert(_wm && _wm->m_packets.size() > iw_index);
	
	if(_wm->m_packets[iw_index].valid)
		return _wm->m_packets[iw_index].packet;
	
	return NULL;
}

Bitmask* WireMatrixEntry::remaining() const
{
	// Annahme: keine Bereichueberschreitung
	assert(_wm && _wm->m_packets.size() > iw_index);
	
	if(_wm->m_packets[iw_index].valid)
		return _wm->m_packets[iw_index].remain; 
	
	return NULL;
}

const Bitmask* WireMatrixEntry::targets() const
{
	// Annahme: keine Bereichueberschreitung
	assert(_wm && _wm->inwires.size() > iw_index);
	// Annahme: keine Bereichueberschreitung
	assert(_wm && _wm->outwires.size() > ow_index);
	// Annahme: Register vorhanden
	assert(_wm->inwires[iw_index]->RegisterOnLine());
	
	return _wm->inwires[iw_index]->RegisterOnLine()->validTargets(_wm->outwires[ow_index]->To()); 
}

